#include <iostream>
#include "histogram.h"
#include "histogram_internal.h"
#include <algorithm>
#include <cmath>


void find_minmax(const std::vector<double>& numbers, double& min, double& max) {
    if (numbers.empty()) return;

    min = numbers[0];
    max = numbers[0];
    for (double num : numbers) {
        if (num < min) min = num;
        if (num > max) max = num;
    }
}

std::vector<size_t> make_histogram(const std::vector<double>& numbers, size_t bin_count) {
    if (numbers.empty() || bin_count == 0) return {};

    double min, max;
    find_minmax(numbers, min, max);

    std::vector<size_t> bins(bin_count);
    double bin_size = (max - min) / bin_count;

    for (double num : numbers) {
        bool found = false;
        for (size_t j = 0; j < bin_count - 1 && !found; ++j) {
            double lo = min + j * bin_size;
            double hi = min + (j + 1) * bin_size;
            if (lo <= num && num < hi) {
                bins[j]++;
                found = true;
            }
        }
        if (!found) {
            bins[bin_count - 1]++;
        }
    }

    return bins;
}

Input input_data() {
    Input in;
    std::cin >> in.number_count;

    in.numbers.resize(in.number_count);
    for (size_t i = 0; i < in.number_count; i++) {
        std::cin >> in.numbers[i];
    }

    std::cin >> in.bin_count;
    return in;
}

void show_histogram_text(const std::vector<size_t>& bins, size_t max_width) {
    if (bins.empty()) return;

    size_t max_count = *std::max_element(bins.begin(), bins.end());
    if (max_count == 0) return;

    for (size_t count : bins) {

        if (count < 100) std::cout << " ";
        if (count < 10) std::cout << " ";

        std::cout << count << "|";


        size_t bar_length = (count * max_width) / max_count;
        for (size_t i = 0; i < bar_length; ++i) {
            std::cout << "*";
        }
        std::cout << std::endl;
    }
}
